from ucca import layer0, layer1


LINKAGE = (layer1.EdgeTags.LinkArgument, layer1.EdgeTags.LinkRelation)


def validate(passage):
    for node in passage.layer(layer0.LAYER_ID).all:
        if not node.incoming:
            yield "Orphan %s terminal (%s) '%s'" % (node.tag, node.ID, node)
        elif len(node.incoming) > 1:
            yield "Reentrant %s terminal (%s) '%s'" % (node.tag, join(node.incoming), node)
    stack = [list(passage.layer(layer1.LAYER_ID).heads)]
    for node in stack[-1]:
        if node.ID != "1.1" and node.tag != layer1.NodeTags.Linkage:
            yield "Extra root (%s)" % node.ID
    visited = set()
    path = []
    path_set = set(path)
    while stack:
        for node in stack[-1]:
            if node in path_set:
                yield "Detected cycle (%s)" % "->".join(n.ID for n in path)
            elif node not in visited:
                visited.add(node)
                path.append(node)
                path_set.add(node)
                stack.append(node.children)
                if node.tag == layer1.NodeTags.Linkage:
                    if node.incoming:
                        yield "Non-root %s node (%s)" % (node.tag, join(node.incoming))
                    non_linkage = [e for e in node if e.tag not in LINKAGE]
                    if non_linkage:
                        yield "%s node with non-linkage children (%s)" % (node.tag, join(non_linkage))
                    if not any(e.tag == layer1.EdgeTags.LinkRelation for e in node):
                        yield "%s node without %s child" % (node.tag, layer1.EdgeTags.LinkRelation)
                incoming = [e for e in node.incoming if not e.attrib.get("remote") and e.tag not in LINKAGE]
                if len(incoming) > 1:
                    yield "Multiple incoming non-remote (%s)" % join(incoming)
                for edge in node:
                    if (edge.tag == layer1.EdgeTags.Punctuation) != (edge.child.tag == layer1.NodeTags.Punctuation):
                        yield "%s edge (%s) with %s child" % (edge.tag, edge, edge.child.tag)
                    if (node.tag == layer1.NodeTags.Punctuation) != (edge.child.tag == layer0.NodeTags.Punct):
                        yield "%s node (%s) with %s child (%s)" % (node.tag, node.ID, edge.child.tag, edge.child.ID)
                break
        else:
            if path:
                path_set.remove(path.pop())
            stack.pop()


def join(incoming):
    return ", ".join(map(str, incoming))



from collections import OrderedDict

from ucca import textutil, layer0, layer1
from ucca.layer1 import EdgeTags, NodeTags


class Construction(object):
    def __init__(self, name, description, criterion, default=False):
        """
        :param name: short name
        :param description: long description
        :param criterion: predicate function to apply to a Candidate, saying if it is an instance of this construction
        :param default: whether this construction is included in evaluation by default
        """
        self.name = name
        self.description = description
        self.criterion = criterion
        self.default = default

    def __str__(self):
        return self.name

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, other):
        return self.name == (other.name if isinstance(other, Construction) else other)


class Candidate(object):
    def __init__(self, edge, reference=None):
        self.edge = edge
        self.out_tags = {e.tag for e in edge.child}
        self.reference = reference
        self._terminals = self._pos = self._dep = self._heads = self._tokens = None

    def _init_terminals(self):
        if self._terminals is None:
            try:
                self._terminals = self.edge.child.get_terminals()
            except (AttributeError, ValueError):
                self._terminals = ()
            if self.reference is not None:
                # noinspection PyTypeChecker
                self._terminals = [self.reference.by_id(t.ID) for t in self._terminals]

    @property
    def remote(self):
        return self.edge.attrib.get("remote", False)

    @property
    def implicit(self):
        return self.edge.child.attrib.get("implicit", False)

    @property
    def excluded(self):
        return self.edge.tag in EXCLUDED_EDGE_TAGS or self.edge.child.tag in EXCLUDED_NODE_TAGS

    @property
    def pos(self):
        if self._pos is None:
            self._init_terminals()
            self._pos = {textutil.Attr.POS(t.tok[textutil.Attr.POS.value]) for t in self._terminals}
        return self._pos

    @property
    def dep(self):
        if self._dep is None:
            self._init_terminals()
            self._dep = {textutil.Attr.DEP(t.tok[textutil.Attr.DEP.value]) for t in self._terminals}
        return self._dep

    @property
    def heads(self):
        if self._heads is None:
            self._init_terminals()
            positions = {t.para_pos for t in self._terminals}
            self._heads = {t for t in self._terminals if int(t.tok[textutil.Attr.HEAD]) not in positions}
        return self._heads

    @property
    def tokens(self):
        if self._tokens is None:
            self._init_terminals()
            self._tokens = {t.text.lower() for t in self._terminals}
        return self._tokens

    def is_primary(self):
        return not self.remote and not self.implicit and not self.excluded

    def is_remote(self):
        return self.remote and not self.implicit and not self.excluded

    def is_predicate(self):
        return self.edge.tag in {EdgeTags.Process, EdgeTags.State} and \
            self.out_tags <= {EdgeTags.Center, EdgeTags.Function, EdgeTags.Terminal} and \
            "to" not in self.tokens


EXCLUDED_EDGE_TAGS = (EdgeTags.Punctuation,
                      EdgeTags.LinkArgument,
                      EdgeTags.LinkRelation,
                      EdgeTags.Terminal)

EXCLUDED_NODE_TAGS = (NodeTags.Linkage,
                      NodeTags.Punctuation,
                      layer0.NodeTags.Word,
                      layer0.NodeTags.Punct)


CONSTRUCTIONS = (
    Construction("primary", "Regular edges", Candidate.is_primary, default=True),
    Construction("remote", "Remote edges", Candidate.is_remote, default=True),
    Construction("aspectual_verbs", "Aspectual verbs",
                 lambda c: c.pos == {"VERB"} and c.edge.tag == EdgeTags.Adverbial),
    Construction("light_verbs", "Light verbs",
                 lambda c: c.pos == {"VERB"} and c.edge.tag == EdgeTags.Function),
    Construction("mwe", "Multi-word expressions",
                 lambda c: c.is_primary() and c.edge.child.tag == NodeTags.Foundational and len(
                     c.edge.child.terminals) > 1),  # inseparable unit
    Construction("pred_nouns", "Predicate nouns",
                 lambda c: "ADJ" not in c.pos and "NOUN" in c.pos and c.is_predicate()),
    Construction("pred_adjs", "Predicate adjectives",
                 lambda c: "ADJ" in c.pos and "NOUN" not in c.pos and c.is_predicate()),
    Construction("expletives", "Expletives",
                 lambda c: c.tokens <= {"it", "there"} and c.edge.tag == EdgeTags.Function),
    # Construction("part_whole", "Part-whole constructions",
    #              lambda c: is_primary(c) and c.edge.child.tag == NodeTags.Foundational and (
    #                  len(c.edge.child.centers) > 1 and
    #                  not c.heads & {t for n in c.edge.child.centers for t in n.get_terminals()})),
    # Construction("classifiers", "Classifier constructions",
    #              lambda c: is_primary(c) and c.edge.child.tag == NodeTags.Foundational and (
    #                  len(c.edge.child.centers) > 1 and c.dep & {"appos"} and
    #                  not c.heads & {t for n in c.edge.child.centers for t in n.get_terminals()})),
)
PRIMARY = CONSTRUCTIONS[0]
EDGE_TYPES_NAME = "edge_types"
EDGE_TYPES = OrderedDict((v, Construction(v, k, lambda c, tag=v: c.edge.tag == tag))
                         for k, v in sorted(EdgeTags.__dict__.items()) if not k.startswith("_"))
CONSTRUCTION_BY_NAME = OrderedDict([(c.name, c) for c in CONSTRUCTIONS] +
                                   [(EDGE_TYPES_NAME, EDGE_TYPES)] + list(EDGE_TYPES.items()))
DEFAULT = OrderedDict((str(c), c) for c in CONSTRUCTIONS if c.default)


def add_argument(argparser, default=True):
    d = list(DEFAULT.keys()) if default else [n for n in CONSTRUCTION_BY_NAME if n not in DEFAULT]
    argparser.add_argument("--constructions", nargs="*", choices=CONSTRUCTION_BY_NAME, default=d, metavar="x",
                           help="construction types to include, out of {%s}" %
                                ",".join(CONSTRUCTION_BY_NAME.keys()))


def get_by_name(name):
    return name if isinstance(name, Construction) else EDGE_TYPES.get(name) or CONSTRUCTION_BY_NAME[name]


def get_by_names(names):
    constructions = [get_by_name(c) for c in names if c != EDGE_TYPES_NAME]
    if EDGE_TYPES_NAME in names:
        constructions += list(EDGE_TYPES.values())
    return constructions or CONSTRUCTIONS


def terminal_ids(passage):
    return {t.ID for t in passage.layer(layer0.LAYER_ID).all}


def diff_terminals(*passages):
    texts = [[t.text for t in p.layer(layer0.LAYER_ID).all] for p in passages]
    return [[t for t in texts[i] if t not in texts[j]] for i, j in ((0, 1), (1, 0))]


def extract_edges(passage, constructions=None, reference=None, verbose=False):
    """
    Find constructions in UCCA passage.
    :param passage: Passage object to find constructions in
    :param constructions: list of constructions to include or None for all
    :param reference: Passage object to get POS tags from (default: `passage')
    :param verbose: whether to print tagged text
    :return: dict of Construction -> list of corresponding edges
    """
    constructions = get_by_names(constructions)
    if reference is not None:
        ids1, ids2 = terminal_ids(passage), terminal_ids(reference)
        assert ids1 == ids2, "Reference passage terminals do not match: %s (%d != %d)\nDifference:\n%s" % (
            reference.ID, len(terminal_ids(passage)), len(terminal_ids(reference)),
            "\n".join(map(str, diff_terminals(passage, reference))))
    if any(not c.default for c in constructions):
        textutil.annotate(passage, verbose=verbose)
    extracted = OrderedDict((c, []) for c in constructions)
    for node in passage.layer(layer1.LAYER_ID).all:
        for edge in node:
            candidate = Candidate(edge, reference=reference)
            for construction in constructions:
                if construction.criterion(candidate):
                    extracted[construction].append(edge)
    # edges = (e for n in l1.all for e in n if e.tag)
    # for edge in edges:
    #     if args.mwe:
    #         pass
    #     if args.part_whole:
    #         pass
    #     if args.classifiers:
    #         pass
    return extracted



"""
The evaluation library for UCCA layer 1.
v1.2
2016-12-25: move common Fs to root before evaluation
2017-01-04: flatten centers, do not add 1 (for root) to mutual
2017-01-16: fix bug in moving common Fs
2018-04-12: exclude punctuation nodes regardless of edge tag
"""
from collections import Counter, defaultdict, OrderedDict
from operator import attrgetter

from ucca import layer0, layer1
from ucca.constructions import extract_edges, get_by_names, PRIMARY, DEFAULT
from ucca.layer1 import EdgeTags, NodeTags

UNLABELED = "unlabeled"
WEAK_LABELED = "weak_labeled"
LABELED = "labeled"

EVAL_TYPES = (LABELED, UNLABELED, WEAK_LABELED)

# Pairs that are considered as equivalent for the purposes of evaluation
EQUIV = ((EdgeTags.Process, EdgeTags.State),
         (EdgeTags.ParallelScene, EdgeTags.Center),
         (EdgeTags.Connector, EdgeTags.Linker),
         (EdgeTags.Function, EdgeTags.Relator))


def flatten_centers(p):
    """
    Whenever there are Cs inside Cs, remove the external C.
    """
    for unit in p.layer(layer1.LAYER_ID).all:
        if unit.tag == NodeTags.Foundational and unit.ftag == EdgeTags.Center and \
                                len(unit.centers) == len(unit.fparent.centers) == 1:
            for e in unit.incoming:
                if e.attrib.get("remote"):
                    e.parent.add(e.tag, unit.centers[0], edge_attrib=e.attrib)
            for e in unit.outgoing:
                unit.fparent.add(e.tag, e.child, edge_attrib=e.attrib)
            unit.destroy()


def move_functions(p1, p2):
    """
    Move any common Fs to the root
    """
    f1, f2 = [{get_yield(u): u for u in p.layer(layer1.LAYER_ID).all
               if u.tag == NodeTags.Foundational and u.ftag == EdgeTags.Function} for p in (p1, p2)]
    for positions in f1.keys() & f2.keys():
        for (p, unit) in ((p1, f1[positions]), (p2, f2[positions])):
            for parent in unit.parents:
                tag = unit.ftag
                parent.remove(unit)
                p.layer(layer1.LAYER_ID).heads[0].add(tag, unit)


def get_text(p, positions):
    l0 = p.layer(layer0.LAYER_ID)
    return [l0.by_position(i).text for i in range(1, len(l0.all) + 1) if i in positions]


def create_passage_yields(p, constructions=None, reference=None, verbose=False):
    """
    :returns dict: Construction ->
                   dict: set of terminal indices (excluding punctuation) ->
                         list of edges of the Construction whose yield (excluding remotes and punctuation) is that set
    """
    yield_tags = OrderedDict()
    for construction, edges in extract_edges(
            p, constructions=constructions, reference=reference, verbose=verbose).items():
        yield_tags[construction] = {}
        for edge in edges:
            yield_tags[construction].setdefault(get_yield(edge.child), []).append(edge.tag)
    return yield_tags


def get_yield(unit):
    try:
        return frozenset(t.position for t in unit.get_terminals(punct=False))
    except ValueError:
        return frozenset()


def print_tags_and_text(p, yield_tags):
    for y, tags in sorted(yield_tags.items(), key=lambda x: min(x[0] or [0])):
        text = " ".join(get_text(p, y))
        print((",".join(sorted(filter(None, tags))) + ": " + text) if tags else text)


def expand_equivalents(tag_set):
    """
    Returns a set of all the tags in the tag set or those equivalent to them
    :param tag_set: set of tags (strings) to expand
    """
    return tag_set.union(t1 for t in tag_set for pair in EQUIV for t1 in pair if t in pair and t != t1)


class Evaluator(object):
    def __init__(self, verbose, constructions, units, fscore, errors):
        """
        :param verbose: whether to print the scores
        :param constructions: names of construction types to include in the evaluation
        :param units: whether to calculate and print the mutual and exclusive units in the passages
        :param fscore: whether to find and return the scores
        :param errors: whether to calculate and print the confusion matrix of errors
        """
        self.verbose = verbose
        self.constructions = list(DEFAULT.values()) + [c for c in get_by_names(constructions)
                                                       if c not in DEFAULT.values()]
        self.units = units
        self.fscore = fscore
        self.errors = errors

        self.mutual = defaultdict(dict)
        self.error_counters = defaultdict(lambda: defaultdict(Counter))

    def find_mutuals(self, m1, m2, eval_type, construction):
        mutual_tags = self.mutual[construction]
        for y in m1.keys() & m2.keys():
            if eval_type == UNLABELED:
                mutual_tags[y] = ()
            else:
                tags = [set(m1[y]), set(m2[y])]
                if eval_type == WEAK_LABELED:
                    tags[0] = expand_equivalents(tags[0])
                intersection = set.intersection(*tags)
                if intersection:  # non-empty intersection
                    mutual_tags[y] = intersection
                elif self.errors:
                    self.error_counters[eval_type][construction][tuple("|".join(sorted(t)) for t in tags)] += 1

    def get_scores(self, p1, p2, eval_type):
        """
        prints the relevant statistics and f-scores. eval_type can be 'unlabeled', 'labeled' or 'weak_labeled'.
        calculates a set of all the yields such that both passages have a unit with that yield.
        :param p1: passage to compare
        :param p2: reference passage object
        :param eval_type: evaluation type to use, out of EVAL_TYPES
        1. UNLABELED: it doesn't matter what labels are there.
        2. LABELED: also requires tag match (if there are multiple units with the same yield, requires one match)
        3. WEAK_LABELED: also requires weak tag match (if there are multiple units with the same yield,
                         requires one match)
        :returns EvaluatorResults object if self.fscore is True, otherwise None
        """
        maps = [defaultdict(dict), create_passage_yields(p2, self.constructions)]
        if p1 is not None:
            maps[0] = create_passage_yields(p1, self.constructions, reference=p2)
            for construction, yield_tags1 in maps[0].items():
                yield_tags2 = maps[1][construction]
                self.find_mutuals(yield_tags1, yield_tags2, eval_type, construction)

        if self.verbose:
            print("Evaluation type: (" + eval_type + ")")

        only = [{c: {y: tags for y, tags in d.items() if y not in self.mutual[c]} for c, d in m.items()} for m in maps]
        if self.verbose and self.units and p1 is not None:
            print("==> Mutual Units:")
            print_tags_and_text(p1, self.mutual[PRIMARY])
            print("==> Only in guessed:")
            print_tags_and_text(p1, only[0][PRIMARY])
            print("==> Only in reference:")
            print_tags_and_text(p2, only[1][PRIMARY])

        res = EvaluatorResults((c, SummaryStatistics(len(self.mutual[c]), len(only[0][c]), len(only[1][c]),
                                                     self.error_counters[eval_type][c])) for c in self.constructions)
        if self.verbose:
            if self.fscore:
                res.print()
            if self.errors and self.error_counters[eval_type]:
                res.print_confusion_matrix()

        return res


class Scores(object):
    def __init__(self, evaluator_results, name=None, evaluation_format=None):
        """
        :param evaluator_results: dict: eval_type -> EvaluatorResults
        :param name: if not UCCA, name of evaluated format
        :param evaluation_format: if not ucca, lowercase string representation of evaluated format
        """
        self.evaluators = dict(evaluator_results)
        self.name = name or "UCCA"
        self.format = evaluation_format or "ucca"

    def average_f1(self, mode=LABELED):
        """
        Calculate the average F1 score across primary and remote edges
        :param mode: LABELED, UNLABELED or WEAK_LABELED
        :return: a single number, the average F1
        """
        return float(self.evaluators[mode].aggregate_default().f1)

    @staticmethod
    def aggregate(scores):
        """
        Aggregate multiple Scores instances
        :param scores: iterable of Scores
        :return: new Scores with aggregated scores
        """
        scores = list(scores)
        evaluators = [s.evaluators for s in scores]
        names = list(set(s.name for s in scores))
        formats = list(set(s.format for s in scores))
        return Scores(((t, EvaluatorResults.aggregate(filter(None, (e.get(t) for e in evaluators))))
                       for t in EVAL_TYPES),
                      name=names[0] if len(names) == 1 else None,
                      evaluation_format=formats[0] if len(formats) == 1 else None)

    def print(self, **kwargs):
        for eval_type in EVAL_TYPES:
            evaluator = self.evaluators.get(eval_type)
            if evaluator:
                print("Evaluation type: (" + eval_type + ")", **kwargs)
                evaluator.print(**kwargs)

    def print_confusion_matrix(self, *args, **kwargs):
        for eval_type in EVAL_TYPES:
            evaluator = self.evaluators.get(eval_type)
            if evaluator:
                evaluator.print_confusion_matrix("Evaluation type: (" + eval_type + ")", *args, **kwargs)

    def fields(self):
        e = self.evaluators[LABELED]
        return ["%.3f" % float(getattr(x, y)) for x in e.results.values() for y in ("p", "r", "f1")]

    def titles(self):
        return self.field_titles(self.evaluators[LABELED].results.keys())

    @staticmethod
    def field_titles(constructions=DEFAULT):
        return ["_".join(((str(x),) if len(constructions) > 1 else ()) + ("labeled", y))
                for x in constructions for y in ("precision", "recall", "f1")]


class EvaluatorResults(object):
    def __init__(self, results, default=None):
        """
        :param results: dict: Construction -> SummaryStatistics
        :param default: map of default constructions (default is primary and remote)
        """
        self.results = OrderedDict(results)
        self.default = default or DEFAULT

    def print(self, **kwargs):
        for construction, stats in self.results.items():
            if len(self.results) > 1:
                print("\n%s:" % construction.description, **kwargs)
            stats.print(**kwargs)
        print(**kwargs)

    def print_confusion_matrix(self, prefix=None, sep=None, **kwargs):
        primary = self.results.get(PRIMARY)
        if primary and primary.errors:
            errors = primary.errors.most_common()
            if sep:
                print(sep.join(("guessed", "ref", "count")), **kwargs)
                for error, freq in errors:
                    print(sep.join(error + (str(freq),)), **kwargs)
            else:
                print("\n%sConfusion Matrix:" % ("" if prefix is None else (prefix + ", ")), **kwargs)
                for error, freq in errors:
                    l1 = max(len(e1) for e1, _ in primary.errors)
                    l2 = max(len(e2) for _, e2 in primary.errors)
                    print("%-*s %-*s %d" % (l1, error[0], l2, error[1], freq), **kwargs)

    @classmethod
    def aggregate(cls, results):
        """
        :param results: iterable of EvaluatorResults
        :return: new EvaluatorResults with aggregates scores
        """
        collected = OrderedDict()
        default = OrderedDict()
        for evaluator_results in results:
            for c, r in evaluator_results.results.items():
                collected.setdefault(c, []).append(r)
            default.update(evaluator_results.default)
        return EvaluatorResults(((c, SummaryStatistics.aggregate(r)) for c, r in collected.items()), default=default)

    def aggregate_default(self):
        """
        Aggregate primary and remote SummaryStatistics in this EvaluatorResults instance
        :return: SummaryStatistics object representing aggregation over primary and remote
        """
        try:
            return SummaryStatistics.aggregate([self.results[c] for c in self.default.values()])
        except KeyError as e:
            raise ValueError("Default constructions missing from evaluation results: " +
                             ", ".join(map(str, self.results.keys()))) from e

    def __bool__(self):
        return bool(self.results)


class SummaryStatistics(object):
    def __init__(self, num_matches, num_only_guessed, num_only_ref, errors=None):
        self.num_matches = num_matches
        self.num_only_guessed = num_only_guessed
        self.num_only_ref = num_only_ref
        self.num_guessed = num_matches + num_only_guessed
        self.num_ref = num_matches + num_only_ref
        self.p = 1.0 if self.num_guessed == 0 else 1.0 * num_matches / self.num_guessed
        self.r = 1.0 if self.num_ref == 0 else 1.0 * num_matches / self.num_ref
        self.f1 = 0.0 if 0.0 in (self.p, self.r) else 2.0 * self.p * self.r / float(self.p + self.r)
        self.errors = errors

    def print(self, **kwargs):
        print("Precision: {:.3} ({}/{})".format(self.p, self.num_matches, self.num_guessed), **kwargs)
        print("Recall: {:.3} ({}/{})".format(self.r, self.num_matches, self.num_ref), **kwargs)
        print("F1: {:.3}".format(self.f1), **kwargs)

    @classmethod
    def aggregate(cls, stats):
        """
        :param stats: iterable of SummaryStatistics
        :return: new SummaryStatistics with aggregated scores
        """
        return SummaryStatistics(*map(sum, [map(attrgetter(attr), stats)
                                            for attr in ("num_matches", "num_only_guessed", "num_only_ref")]),
                                 Counter({k: sum(s.errors.get(k, 0) for s in stats)
                                          for k in set.union(*[set(s.errors or ()) for s in stats])}))


def evaluate(guessed, ref, converter=None, verbose=False, constructions=DEFAULT,
             units=False, fscore=True, errors=False, normalize=True, **kwargs):
    """
    Compare two passages and return requested diagnostics and scores, possibly printing them too.
    NOTE: since normalize=True by default, this method is destructive: it modifies the given passages before evaluation.
    :param guessed: Passage object to evaluate
    :param ref: reference Passage object to compare to
    :param converter: optional function to apply to passages before evaluation
    :param verbose: whether to print the results
    :param constructions: names of construction types to include in the evaluation
    :param units: whether to evaluate common units
    :param fscore: whether to compute precision, recall and f1 score
    :param errors: whether to print the mistakes
    :param normalize: flatten centers and move common functions to root before evaluation - modifies passages
    :return: Scores object
    """
    del kwargs
    if converter is not None:
        guessed = converter(guessed)
        ref = converter(ref)
    if normalize:
        for passage in (guessed, ref):
            flatten_centers(passage)  # flatten Cs inside Cs
        move_functions(guessed, ref)  # move common Fs to be under the root

    evaluator = Evaluator(verbose, constructions, units, fscore, errors)
    return Scores((evaluation_type, evaluator.get_scores(guessed, ref, evaluation_type))
                  for evaluation_type in EVAL_TYPES)



#! /usr/bin/python3

import argparse
import os
import pickle
import sqlite3
from xml.etree.ElementTree import ElementTree, tostring, fromstring

import ucca.convert
from ucca.textutil import indent_xml

desc = """Parses an XML in UCCA site format.

The input can be given as either an XML file or a DB file with passage ID
and user name, and the output is either the standard format XML or
a pickled object.
Possible input methods are using a DB file with pid and user, which gets the
annotation of the specified user for the specified passage from teh DB file,
or using filenames of a site-formatted XML file.

"""


def site2passage(filename):
    """Opens a file and returns its parsed Passage object"""
    with open(filename, encoding="utf-8") as f:
        etree = ElementTree().parse(f)
    return ucca.convert.from_site(etree)


def db2passage(handle, pid, user):
    """Gets the annotation of user to pid from the DB handle - returns a passage"""
    handle.execute("SELECT id FROM users WHERE username=?", (user,))
    uid = handle.fetchone()[0]
    handle.execute("SELECT xml FROM xmls WHERE paid=? AND uid=? " +
                   "ORDER BY ts DESC", (pid, uid))
    raw_xml = handle.fetchone()[0]
    return ucca.convert.from_site(fromstring(raw_xml))


def outfile(source, target, suffix):
    return os.path.join(target, os.path.splitext(source)[0] + suffix) if os.path.isdir(target) else target


def main(args):
    if args.filenames:
        passages = ((filename, site2passage(filename)) for filename in args.filenames)
    else:
        conn = sqlite3.connect(args.db)
        c = conn.cursor()
        passages = ((pid, db2passage(c, pid, args.user)) for pid in args.pids)

    for filename, passage in passages:
        if args.binary:
            with open(outfile(filename, args.binary, ".pickle"), "wb") as binf:
                pickle.dump(passage, binf)
        else:
            root = ucca.convert.to_standard(passage)
            output = indent_xml(tostring(root).decode())
            if args.outfile:
                with open(outfile(filename, args.outfile, ".xml"), "w", encoding="utf-8") as outf:
                    outf.write(output)
            else:
                print(output)


def check_illegal_combinations(args):
    if args.db and args.filenames:
        argparser.error("Only one source, XML or DB file, can be used")
    if (not args.db) and (not args.filenames):
        argparser.error("Must specify one source, XML or DB file")
    if args.db and not (args.pids and args.user):
        argparser.error("Must specify a username and a passage ID when " +
                        "using DB file option")
    if (args.pids or args.user) and not args.db:
        argparser.error("Cannot use user and passage ID options without DB file")
    return args


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="*", help="XML file name to convert")
    argparser.add_argument("-o", "--outfile", help="output file for standard XML")
    argparser.add_argument("-b", "--binary", help="output file for binary pickle")
    argparser.add_argument("-d", "--db", help="DB file to get input from")
    argparser.add_argument("-p", "--pids", nargs="*", type=int, help="PassageIDs to query DB")
    argparser.add_argument("-u", "--user", help="Username to DB query")
    main(check_illegal_combinations(argparser.parse_args()))



#!/usr/bin/env python

import os
import sys

from setuptools import setup, find_packages

try:
    this_file = __file__
except NameError:
    this_file = sys.argv[0]
os.chdir(os.path.dirname(os.path.abspath(this_file)))

try:
    import pypandoc
    try:
        pypandoc.convert_file("README.md", "rst", outputfile="README.rst")
    except (IOError, ImportError, RuntimeError):
        pass
    long_description = pypandoc.convert_file("README.md", "rst")
except (IOError, ImportError, RuntimeError):
    long_description = ""


setup(name="UCCA",
      version="1.0.64",
      install_requires=["numpy", "spacy==2.0.11", "requests", "tqdm"],
      extras_require={"visualize": ["matplotlib", "networkx"],
                      "distances": ["distances", "zss", "munkres"]},
      description="Universal Conceptual Cognitive Annotation",
      long_description=long_description,
      author="Daniel Hershcovich",
      author_email="danielh@cs.huji.ac.il",
      url="https://github.com/huji-nlp/ucca",
      classifiers=[
          "Development Status :: 4 - Beta",
          "Intended Audience :: Science/Research",
          "Programming Language :: Python :: 3.6",
          "Topic :: Text Processing :: Linguistic",
          "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
      ],
      packages=find_packages(),
      )



import os
from argparse import ArgumentParser

import matplotlib.pyplot as plt
from tqdm import tqdm

from ucca import visualization
from ucca.ioutil import get_passages_with_progress_bar

if __name__ == "__main__":
    argparser = ArgumentParser(description="Visualize the given passages as graphs.")
    argparser.add_argument("passages", nargs="+", help="UCCA passages, given as xml/pickle file names")
    argparser.add_argument("--tikz", action="store_true", help="print tikz code rather than showing plots")
    argparser.add_argument("--out-dir", help="directory to save figures in (otherwise displayed immediately)")
    args = argparser.parse_args()

    if args.out_dir:
        os.makedirs(args.out_dir, exist_ok=True)
    for passage in get_passages_with_progress_bar(args.passages, desc="Visualizing"):
        if args.tikz:
            tikz = visualization.tikz(passage)
            if args.out_dir:
                with open(os.path.join(args.out_dir, passage.ID + ".tikz.txt"), "w") as f:
                    print(tikz, file=f)
            else:
                with tqdm.external_write_mode():
                    print(tikz)
        else:
            visualization.draw(passage)
            if args.out_dir:
                plt.savefig(os.path.join(args.out_dir, passage.ID + ".png"))
            else:
                plt.show()
